! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!+
! Module: protonscale_mod
!
! Authors:  Brendan Perry (NASA/GSFC), brendan.h.perry@nasa.gov (SAS compliance)
!           Steven L. Snowden (NASA/GSFC), steven.l.snowden@nasa.gov (original)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: protonscale_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!
!  Returns the average SP flux and BACKSCAL keyword for a region
!
!  proton-scale uses a mask image created by mos-spectra to find
!  the average SP flare intensity for that region, and returns the
!  BACKSCAL keyword from the spectrum for that region.
!
! Usage:
!   data directory
!   mode: 1 -> single region
!         2 -> multiple regions
!   mode = 1
!       MOS1: 1, MOS2: 2, pn: 3
!       region image file name
!       region spectrum file name
!       Example: 
!             protonscale mode=1 maskfile=region-image.fits specfile=region-spec.fits
!   mode = 2
!       filename with region information
!       Example:  protonscale mode=2 spfile=spregion.txt
!
module protonscale_mod

  use cal
  use dal
  use param 
  use errorHandling
  use esas_mod      ! in goflib, formerly in subsn (SS)
  use gofutils_mod  ! in goflib, formerly in marq_mod
  use gofio_mod     ! in goflib, formerly in subsn (SS)

  implicit none

  private

  character(len=*), parameter :: &
       fimagepar = "maskfile", &  ! mask image filename
       fspecpar  = "specfile", &  ! mask spectral filename
       modepar   = "mode", &      ! single file or region list
       sppar     = "spfile"       ! text regions file 

  public protonscale

contains

  !---------------------------------------------------------------------
  subroutine protonscale
    !---------------------------------------------------------------------
    character(len=256) :: mask_F, spec_F, sp_F
    integer :: i, ios, lun, mode, n
    real :: back, backl(100), spsum, spsuml(100)

    ios = 0
    mode = intParameter(modepar)
    if (mode .eq. 1) then     
       mask_F = stringParameter(fimagepar)
       spec_F = stringParameter(fspecpar)
       !
       call spscale(mask_F,spec_F,back,spsum) ! in esas_mod in goflib
       !
       write(errstr,*) "Area             : ",back
       call message(AppMsg,Always)
       write(errstr,*) "Ave Flux         : ",spsum
       call message(AppMsg,Always)
       write(errstr,*) "Scaled Ave Flux  : ",1.0E05 * spsum * back
       call message(AppMsg,Always)  
    else
       sp_F = stringParameter(sppar)
       write(errstr,*) "SP text file list: ",trim(adjustl(sp_F))
       call message(AppMsg,Always)
       lun = GetLun() ! in esas_mod in goflib
       open(unit=lun,form='formatted',status='old',&
            file=trim(adjustl(sp_F)),iostat=ios)
       if (ios .ne. 0) then
          call error ("badTextFile",&
               "Input region text file does not exist or is unreadable")
       endif
       i = 0
       do while (ios .eq. 0)
          read(lun,'(A)',iostat=ios) mask_F
          read(lun,'(A)',iostat=ios) spec_F
          mask_F = trim(adjustl(mask_F))
          spec_F = trim(adjustl(spec_F))
          i = i + 1
          if (i .gt. 100) then ! change this???!!!
             call warning('maxMasks100',"Maximum masks/specs is gt 100, will do only the first 100")
             exit
          end if
          if(ios .eq. 0) then
             write(errstr,*) "current mask/spec file : ",&
                  trim(adjustl(mask_F)),"/",trim(adjustl(spec_F))
             call message(AppMsg,Noisy)  
             !
             call spscale(mask_F,spec_F,backl(i),spsuml(i))
             !
          endif
       enddo
       n = i - 1
       write(errstr,*)'  '
       call message(AppMsg,Always)
       write(errstr,*)'         Solid   Soft Proton'
       call message(AppMsg,Always)
       write(errstr,*)'         Angle     Scaling'
       call message(AppMsg,Always)
       write(errstr,*)'  '
       do i=1,n
          write(errstr,2000)i,backl(i),backl(i)*spsuml(i)/(backl(1)*spsuml(1))
2000      format(1x,i5,2f10.4)
          call message(AppMsg,Always)
       enddo
    endif

    return

  end subroutine protonscale

end module protonscale_mod
